#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
# @Author  : wangzc
# @Date    : 2024/8/2 16:44
# @File    : device_base.py
# @Description: 
"""


class BaseDevice(object):

    def __init__(self, bustype: str, channel: int, bitrate: int):
        self.bustype = bustype
        self.channel = channel
        self.bitrate = bitrate
        self.running = False
        self.status = None




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
# @Author  : wangzc
# @Date    : 2024/7/30 14:48
# @File    : UsbManager.py
# @Description:
"""
import threading

from .usb_trigger import DeviceNotifier
import json
import asyncio


class UsbManager(object):

    # canalystii  VID_04D8&PID_0053
    # vector      VID_1248&PID_1080
    # neovi       VID_093C&PID_1000

    def __init__(self, logger):
        # 根据usb状态拿设备信息
        self.logger = logger
        self.device_pool: dict[str, int] = {}
        self.notifier = DeviceNotifier(self.device_pool)
        self.notifier.getListNow()
        self.startListen()
        # asyncio.run(self.notifier.register_and_wait())

    def getDevice(self, device_type: str, channel: int) -> bool:
        return device_type in self.device_pool and self.device_pool[device_type] - 1 >= channel

    def getList(self, device_type: str):
        # print(device_type)
        return json.dumps(self.notifier.getListNow())

    def startListen(self):
        thread = threading.Thread(target=self.notifier.register_and_wait)
        thread.start()




from enum import Enum


class ServerError(Enum):

    # 通用错误
    EXAMPLE_ERROR = (20000, "ERROR")

    # CAN
    CAN_DEVICE_NO_FOUND = (20500, "can设备不存在")
    IMPOSSIBLE_DBC_UUID = (20501, "该DBC文件不能删除，请联系管理员")
    # Hardware


    def __init__(self, code, msg):
        self.code = code
        self.msg = msg


class ServiceException(Exception):
    def __init__(self, code, msg):
        self.code = code
        self.msg = msg



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
# @Author  : wangzc
# @Date    : 2024/7/30 14:48
# @File    : device_manager.py
# @Description: 
"""
import can

from .device_can import CanDevice
from ..utils.error import ServiceException, ServerError


class DeviceManager(object):

    # canalystii
    def __init__(self, logger):
        self.logger = logger
        self.usb_status = None
        # 根据usb状态拿设备信息
        self.can_list: list = []

    def findCanDevice(self, device_type: str, channel: int) -> CanDevice:
        for device in self.can_list:
            if isinstance(device, CanDevice) and device.bustype == device_type and device.channel == channel:
                return device
        raise ServiceException(ServerError.CAN_DEVICE_NO_FOUND.code,
                               ServerError.CAN_DEVICE_NO_FOUND.msg)

    def startCanDevice(self, device_type: str, channel: int, bitrate: int) -> bool:
        try:
            self.logger.info('启动can设备:: 设备类型 - ' + device_type + ' | channel - ' + str(channel))
            tmp = CanDevice(device_type, channel, bitrate)
            tmp.startCan()
            self.can_list.append(tmp)
            return True
        except Exception as ex:
            # self.logging.exception(str(ex))
            print(str(ex))

    def stopCanDevice(self, device_type: str, channel: int) -> bool:
        try:
            self.logger.info('关闭can设备:: 设备类型 - ' + device_type + ' | channel - ' + str(channel))
            devicecan = self.findCanDevice(device_type, channel)
            return devicecan.stopCan()
        except Exception as ex:
            # self.logging.exception(str(ex))
            print(str(ex))

    def sendCanMessage(self, device_type: str, channel: int, arbitration_id: int, data, is_extended_id=False, is_remote_frame=False):
        try:
            self.logger.info('发送can消息:: 设备类型 - ' + device_type + ' | channel - ' + str(channel))
            self.logger.info('arbitration_id: ' + str(arbitration_id) + ' data: ' + str(data) + ' is_extended_id: ' + str(is_extended_id) + ' is_remote_frame: ' + str(is_remote_frame))
            devicecan = self.findCanDevice(device_type, channel)
            return devicecan.sendCan(arbitration_id, data, is_extended_id, is_remote_frame)
            # .sendCan(arbitration_id=0x123, data=[0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08], is_extended_id=False)
        except Exception as ex:
            # self.logging.exception(str(ex))
            print(str(ex))

    def recvCanMessage(self, device_type: str, channel: int, num: int) -> str:
        devicecan = self.findCanDevice(device_type, channel)
        return devicecan.recvCan(num)



import asyncio
import time
import win32gui, win32con, win32api
import win32gui_struct

GUID_DEVINTERFACE_USB_DEVICE = "{A5DCBF10-6530-11D2-901F-00C04FB951ED}"
class DeviceNotifier:

    # 供应商ID(VID)和产品识别码(PID)
    device_identifier = {
        "canalystii": "VID_04D8&PID_0053",
        "vector": "VID_1248&PID_1080",
        "neovi": "VID_093C&PID_1000",
    }

    def __init__(self, device_pool):
        self.device_pool: dict[str, int] = device_pool

    def compare_dicts(self,dict1: dict[str, int], dict2: dict[str, int]) -> bool:
        return dict1 == dict2

    def getListNow(self) -> dict[str, int]:
        device_now = {}
        import win32com.client
        wmi = win32com.client.GetObject("winmgmts:")
        for usb in wmi.InstancesOf("win32_usbcontrollerdevice"):
            for device_name, vid_pid in self.device_identifier.items():
                if vid_pid in usb.Dependent:
                    # Increment the count for the matched device in the device_pool
                    device_now[device_name] = device_now.get(device_name, 0) + 1
                    break
        if not self.compare_dicts(self.device_pool,device_now):
            # from copy import deepcopy
            # self.device_pool = deepcopy(device_now)
            self.copy_dict_contents(self.device_pool,device_now)
        return self.device_pool

    def copy_dict_contents(self, a, b):
        a.clear()  # 清空字典 a
        for key, value in b.items():
            a[key] = value  # 将 b 的内容复制到 a

    def OnDeviceChange(self, hwnd, msg, wp, lp):
        try:
            info = win32gui_struct.UnpackDEV_BROADCAST(lp)
            if info is None:
                return True
            if wp == win32con.DBT_DEVICEARRIVAL:
                # print("插入usb")
                for device_name, vid_pid in self.device_identifier.items():
                    if vid_pid in info.name:
                        # Increment the count for the matched device in the device_pool
                        self.device_pool[device_name] = self.device_pool.get(device_name, 0) + 1
                        # print("增添了：" + device_name)
                        break
                return True
            elif wp == win32con.DBT_DEVICEREMOVECOMPLETE:
                # print("拔出usb")
                for device_name, vid_pid in self.device_identifier.items():
                    if vid_pid in info.name:
                        # Decrement the count for the matched device in the device_pool
                        # print("移除了：" + device_name)
                        if device_name in self.device_pool:
                            self.device_pool[device_name] = max(0, self.device_pool[device_name] - 1)
                        break
            return True
        except Exception as e:
            print(f"Error processing device change: {e}")
            return True


    def register_and_wait(self):
        wc = win32gui.WNDCLASS()
        wc.lpszClassName = "test_devicenotify"
        wc.style = win32con.CS_GLOBALCLASS | win32con.CS_VREDRAW | win32con.CS_HREDRAW
        wc.hbrBackground = win32con.COLOR_WINDOW + 1
        wc.lpfnWndProc = {
            win32con.WM_DEVICECHANGE: self.OnDeviceChange
        }
        class_atom = win32gui.RegisterClass(wc)
        hwnd = win32gui.CreateWindow(
            wc.lpszClassName,
            "Testing some devices",
            win32con.WS_CAPTION,
            100,
            100,
            900,
            900,
            0,
            0,
            0,
            None,
        )

        filter = win32gui_struct.PackDEV_BROADCAST_DEVICEINTERFACE(GUID_DEVINTERFACE_USB_DEVICE)
        hdev = win32gui.RegisterDeviceNotification(
            hwnd, filter, win32con.DEVICE_NOTIFY_WINDOW_HANDLE
        )

        while True:
            win32gui.PumpWaitingMessages()
            # await asyncio.sleep(0.01)
            time.sleep(0.01)

        win32gui.DestroyWindow(hwnd)
        win32gui.UnregisterClass(wc.lpszClassName, None)







#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
# @Author  : wangzc
# @Date    : 2024/8/2 16:43
# @File    : device_can.py
# @Description: 
"""
import can
from can import Message

from .device_base import BaseDevice


class CanDevice(BaseDevice):

    # canalystii
    def __init__(self, bustype: str, channel: int, bitrate: int):
        super().__init__(bustype, channel, bitrate)  # 调用基类的初始化方法
        self.bus = None


    # no init
    def startCan(self):
        self.bus = can.interface.Bus(
            bustype=self.bustype,
            channel=self.channel,
            bitrate=self.bitrate
        )  # 初始化CAN1通道用来发送
        self.running = True
        return True

    def stopCan(self):
        self.bus.shutdown()
        self.running = False
        return True

    def sendCan(self, arbitration_id: int, data, is_extended_id=True, is_remote_frame=False):
        msg = can.Message(
            arbitration_id=arbitration_id,
            is_extended_id=is_extended_id,
            is_remote_frame=is_remote_frame,
            data=data,
            check=True
        )
        self.bus.send(msg)
        return True

    def recvCan(self, num=0):
        # Message(
        #     channel=channel,
        #     timestamp=timestamp,
        #     arbitration_id=raw_msg.can_id,
        #     is_extended_id=raw_msg.extended,
        #     is_remote_frame=raw_msg.remote,
        #     dlc=raw_msg.data_len,
        #     data=bytes(raw_msg.data),
        # ),
        return self.bus.recv(num)


import codecs
import os
from setuptools import setup, find_packages

# these things are needed for the README.md show on pypi (if you dont need delete it)
here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

# you need to change all these
VERSION = '0.0.6'
DESCRIPTION = 'DESCRIPTION'
LONG_DESCRIPTION = 'LONG_DESCRIPTION'

setup(
    name="gugurelay",
    version=VERSION,

    install_requires=[
        'python-can',
        'python-can[canalystii]',
        'pywin32',
        'pywin32-ctypes',
    ],

    author="wangzc",
    author_email="",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    # packages=find_packages(where='src'),  # Specify the source directory
    # package_dir={'': 'src'},  # Maps the root package to the src directory
    packages=find_packages(),
    keywords=['python', 'can'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        # "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)
